import pytest

from fastmcp import Client
from pydantic import AnyUrl


@pytest.mark.asyncio
@pytest.mark.parametrize(
    "prop,expected",
    [
        ("analogValue/1/presentValue", "5.0"),
        ("binaryValue/1/presentValue", "1"),
    ],
)
async def test_read_property(server, mcp, prop, expected):
    """Test read_property resource."""
    async with Client(mcp) as client:
        result = await client.read_resource(
            AnyUrl(f"udp://{server.host}:{server.port}/{prop}")
        )
        assert len(result) == 1
        assert result[0].text == expected


@pytest.mark.asyncio
async def test_write_property(server, mcp):
    """Test write_property tool."""
    async with Client(mcp) as client:
        result = await client.call_tool(
            "write_property",
            {
                "host": server.host,
                "port": server.port,
                "obj": "analogValue,1",
                "prop": "presentValue",
                "data": "11",
            },
        )
        assert len(result.content) == 1
        assert "succedeed" in result.content[0].text


@pytest.mark.asyncio
async def test_who_is(server, mcp):
    """Test who_is tool."""
    async with Client(mcp) as client:
        result = await client.call_tool(
            "who_is",
            {
                "low": 999,
                "high": 1001,
            },
        )
        assert not result.is_error


@pytest.mark.asyncio
async def test_who_has(server, mcp):
    """Test who_has tool."""
    async with Client(mcp) as client:
        result = await client.call_tool(
            "who_has",
            {"low": 999, "high": 1001, "obj": "analogValue,1"},
        )
        assert not result.is_error


@pytest.mark.asyncio
async def test_help_prompt(mcp):
    """Test help prompt."""
    async with Client(mcp) as client:
        result = await client.get_prompt("bacnet_help", {})
        assert len(result.messages) == 5


@pytest.mark.asyncio
async def test_error_prompt(mcp):
    """Test error prompt."""
    async with Client(mcp) as client:
        result = await client.get_prompt(
            "bacnet_error", {"error": "Could not read data"}
        )
        assert len(result.messages) == 2



from pydantic import BaseModel
from pydantic_settings import (
    BaseSettings,
    JsonConfigSettingsSource,
    PydanticBaseSettingsSource,
    SettingsConfigDict,
)


class Auth(BaseModel):
    key: str | None = None


class BACnet(BaseModel):
    host: str = "127.0.0.1"
    port: int = 47808


class Device(BaseModel):
    name: str
    host: str
    port: int


class Settings(BaseSettings):
    auth: Auth = Auth()
    bacnet: BACnet = BACnet()
    devices: list[Device] = []
    model_config = SettingsConfigDict(
        env_file=".env",
        env_file_encoding="utf-8",
        env_nested_delimiter="__",
        env_prefix="BACNET_MCP_",
    )

    @classmethod
    def settings_customise_sources(
        cls,
        settings_cls: type[BaseSettings],
        init_settings: PydanticBaseSettingsSource,
        env_settings: PydanticBaseSettingsSource,
        dotenv_settings: PydanticBaseSettingsSource,
        file_secret_settings: PydanticBaseSettingsSource,
    ) -> tuple[PydanticBaseSettingsSource, ...]:
        return (
            init_settings,
            env_settings,
            dotenv_settings,
            file_secret_settings,
            JsonConfigSettingsSource(settings_cls, json_file="devices.json"),
        )



from bacnet_mcp.settings import Settings


def get_device(
    settings: Settings,
    name: str | None = None,
    host: str | None = None,
    port: int | None = None,
) -> tuple[str, int]:
    """Find a device by name or return the default settings."""
    if name:
        for x in settings.devices:
            if x.name == name:
                return x.host, x.port
        raise RuntimeError("Device not found")
    return (
        host if host is not None else settings.bacnet.host,
        port if port is not None else settings.bacnet.port,
    )



import asyncio
import os

from contextlib import suppress
from openai import AsyncOpenAI
from pydantic import BaseModel
from pydantic_settings import BaseSettings, SettingsConfigDict

from bacnet_mcp import BACnetMCP


class BACnetMCPRunner:
    def __init__(self):
        self._mcp = BACnetMCP()
        self._task: asyncio.Task | None = None

    async def __aenter__(self):
        self._task = asyncio.create_task(self._mcp.run_async(transport="http"))
        await asyncio.sleep(3.0)
        return self._mcp

    async def __aexit__(self, exc_type, exc, tb):
        if self._task:
            self._task.cancel()
            with suppress(asyncio.CancelledError):
                await self._task


class Auth(BaseModel):
    key: str | None = None


class Server(BaseModel):
    name: str = "bacnet-mcp"
    url: str = "http://127.0.0.1:8000/mcp"


class Settings(BaseSettings):
    auth: Auth = Auth()
    server: Server = Server()
    model_config = SettingsConfigDict(
        env_file=".env", env_file_encoding="utf-8", env_nested_delimiter="__"
    )


settings = Settings()

client = AsyncOpenAI(api_key=os.environ.get("OPENAI_API_KEY"))


async def create_response(msg):
    print(f"Running: {msg}")
    return await client.responses.create(
        model="gpt-4.1",
        tools=[
            {
                "type": "mcp",
                "server_label": settings.server.name,
                "server_url": settings.server.url,
                "allowed_tools": ["read_property", "write_property"],
                "require_approval": "never",
                **(
                    {"headers": {"Authorization": f"Bearer {settings.auth.key}"}}
                    if settings.auth.key
                    else {}
                ),
            }
        ],
        input=msg,
    )


async def main():
    async with BACnetMCPRunner():
        for prompt in [
            "Read the presentValue property of analogInput,1.",
            "Write the value 42 to analogValue instance 1.",
            "Set the presentValue of binaryOutput 3 to True.",
        ]:
            resp = await create_response(prompt)
            print(resp.output_text)


if __name__ == "__main__":
    asyncio.run(main())
